#include <stdio.h>
#include <spu_intrinsics.h>
#include <spu_mfcio.h>
#include <tnw_spu.h>

#include "commands.h"

#define READ_MAIL() spu_readch(SPU_RdInMbox)
#define WRITE_MAIL(a) spu_writech(SPU_WrOutMbox,(a))


#define WAIT_DMA(tag)    \
spu_writech(MFC_WrTagMask, (1 << tag)); \
(void)spu_mfcstat(MFC_TAG_UPDATE_ALL);


unsigned int recv_buf0[4096] __attribute__((aligned(128)));
unsigned int test_buf0[4096] __attribute__((aligned(128)));

tnw_rx_t rx;

#define MSGSIZE (1024)


int command_loop();

void fill_message(unsigned int *p, uint n, uint link, uint channel) {

  uint i;  
 
  for (i=0;i<n/4;i+=4) {
    p[i]=link;
    p[i+1]=channel;
    p[i+2]=i+2;
    p[i+3]=i+3;
    
  }
}

void recv_send_sb() {

  uint lnk;
  uint ch;
  uint nidx;
  uint iter = 128;
  uint dma_tag;
  
  tnw_rx_t *rx_ptr;

  lnk = READ_MAIL();
  ch = READ_MAIL();
  
  printf("recv... on lnk %d ch %d spu physical id = %d\n", lnk, ch, 
	 tnw_get_spe_phys_id());
  
  dma_tag = mfc_tag_reserve();

  /* configure credit base */
  tnw_credit_base(lnk, ch, 0);
  /* configure for receiving on lnk, ch, with this rx handler */
  tnw_init_rx(&rx, lnk, ch);
  
  
  /* prepare the message */
  fill_message(test_buf0, MSGSIZE, lnk, ch);
  
  nidx = 0;

  while(iter) {
  
    /* credit with ctag nidx   */
    tnw_credit(&rx, (uint)recv_buf0, MSGSIZE, nidx);
  
    /* wait the previous put */
    tnw_wait_put(dma_tag);
    
    /*
    void tnw_put(unsigned int phy, unsigned int channel, void *txbuf, 
          unsigned int offset, unsigned int size, unsigned int dmatag);
    */
        
    tnw_put(lnk, ch, test_buf0, 0, MSGSIZE, dma_tag);
    
    /* wait for the notify to arrive, rx handler rx, ctag  nidx */
    
    tnw_wait_notify(&rx, nidx);
    
    printf("exit wait nidx = %d\n", nidx);
    
    nidx++;
    nidx = nidx%16;
    iter--;
  }

  iter = 128;



  /* now we use the dynamic allocation of rx handler */
  
  rx_ptr = tnw_rx_alloc();

  /* init again since we changed rx hadler */  
  tnw_init_rx(rx_ptr, lnk, ch);

  printf("now with dynamic notify buffer\n");

  while(iter) {
  
    
    tnw_credit(rx_ptr, (uint)recv_buf0, MSGSIZE, nidx);
   
    /* wait the previous put */
    tnw_wait_put(dma_tag);
    
    /*
    void tnw_put(unsigned int phy, unsigned int channel, void *txbuf, 
          unsigned int offset, unsigned int size, unsigned int dmatag);
    */

    
    tnw_put(lnk, ch, test_buf0, 0, MSGSIZE, dma_tag);
    
        
    tnw_wait_notify(rx_ptr,  nidx);
    
    printf("exit wait nidx = %d\n", nidx);
    
    nidx++;
    nidx = nidx%16;
    iter--;
  }
  
  /* deallocation of notify buffer */
  tnw_rx_free(rx_ptr);
  
  mfc_tag_release(dma_tag);
  
  printf("ciaooo\n");
}

void send() {

  uint lnk;
  uint ch;
  
  uint iter = 1;
  uint dma_tag;
  
  

  lnk = READ_MAIL();
  ch = READ_MAIL();
  
    
  dma_tag = mfc_tag_reserve();

  
  
  
  /* prepare the message */
  fill_message(test_buf0, MSGSIZE, lnk, ch);
  
  

  while(iter) {
  
    /* wait the previous put */
    tnw_wait_put(dma_tag);
    
    /*
    void tnw_put(unsigned int phy, unsigned int channel, void *txbuf, 
          unsigned int offset, unsigned int size, unsigned int dmatag);
    */
        
    tnw_put(lnk, ch, test_buf0, 0, 128, dma_tag);
    
    /* wait for the notify to arrive, rx handler rx, ctag  nidx */
    
    
    iter--;
  }
  
  mfc_tag_release(dma_tag);

}

int main(unsigned long long id, unsigned long long argp, 
	 unsigned long long envp) {
  
  
  
  tnw_init(id);
  
  command_loop();
  
  tnw_finalize();
	
	return 0;
}


int command_loop() {
  
  uint cmd;
    
  for(;;) {

    cmd=READ_MAIL();
    
    switch (cmd) {

    case SPU_CMD_EXIT:
      return (0);
      break;
             
    case SPU_CMD_RECV:
      recv_send_sb();
      break;

    case SPU_CMD_SEND:
      send();
      break;
      
    default:
      break;
      
    }
    
  }
  return 1;
}

