/*
 * Copyright (c) 2009 Andrea Nobile
 * 
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */


#ifndef TNW_H
#define TNW_H

#include <inttypes.h>

#define MDIOBUSY 0x20000000

/*tx and rx */
#define DCR_SLV_RX0  (0x00 << 4)
#define DCR_SLV_TX0  (0x60 << 4)

/* notify base */
#define DCR_SLV_NY0  (0x01 << 4)

/* credit fifo */
#define DCR_SLV_CF0  (0x02 << 4)


/* credit base */
#define DCR_SLV_CB0  (0x03 << 4)

/* TXLINK DCR Register Addresses */
#define TX_RA_MDIO_R  0x3
#define TX_RA_MDIO_W  0x4

#define TNW_CREDIT(recv_addr, size, nindex) \
  ( ((nindex & 0xf) << 28) | ((((size) >>7) & 0xfff) << 16) | (((recv_addr) >> 7) & 0xffff) )

#define TNW_CREDIT_REGADDR(phy, channel) \
  ( DCR_SLV_CF0 + ((phy) << 8) + (channel) )


#define TNW_PUT_ADDR(phy, channel) \
  ( _tnw_eas.iwc_ea + (phy)*(1 << 21) + (channel)*(1 << 18) )


typedef struct {
  uint32_t dcr_ea;
  uint32_t iwc_ea;
} tnw_device_eas;


typedef struct 
{
  uint32_t bp;
  uint32_t nc;
} tnw_nc_pos_t;


typedef struct 
{
  unsigned long id;
  unsigned int phys_id;
  tnw_device_eas tnw_eas;
  unsigned int lnk[6];
} tnw_init_params_t;



#endif
