#ifndef TNW_SPU_H
#define TNW_SPU_H

#include <spu_intrinsics.h>
#include <spu_mfcio.h>
#include "tnw.h"


typedef struct {
  volatile vector unsigned short buf;
  unsigned int phy;
  unsigned int ch;
  unsigned short state;
} tnw_rx_t  __attribute__((aligned(128)));

void tnw_init(unsigned long long id);
void tnw_finalize();

/* low level access */
void tnw_reg_write(unsigned int addr, unsigned int data);
unsigned int tnw_reg_read(unsigned int addr);

void tnw_rx_write(unsigned int phy, unsigned int reg, unsigned int data);
unsigned int tnw_rx_read(unsigned int phy, unsigned int reg);

void tnw_tx_write(unsigned int phy, unsigned int reg, unsigned int data);
unsigned int tnw_tx_read(unsigned int phy, unsigned int reg);


unsigned int tnw_mdio_read(unsigned int phy, unsigned int reg);
void tnw_mdio_write(unsigned int phy, unsigned int reg, unsigned int data);

/* notify buffer & state info allocation/free */
tnw_rx_t * tnw_rx_alloc();
void tnw_rx_free(tnw_rx_t *rx);

/* credit & notify configuration  */
void tnw_credit_base(unsigned int phy, unsigned int channel, unsigned int addr);
void tnw_init_rx(tnw_rx_t *rx, unsigned int phy, unsigned int channel);

/* returns 1 if notify arrived, 0 if not arrived */
unsigned int tnw_test_notify(tnw_rx_t *rx, unsigned int ctag);
void tnw_wait_notify(tnw_rx_t *rx, unsigned int ctag);

/* give credit */ 
void tnw_credit(tnw_rx_t *rx, unsigned int addr, 
		unsigned int size, unsigned int ctag);

/* put data to tnw */
void tnw_put(unsigned int phy, unsigned int channel, void *txbuf, 
	     unsigned int offset, unsigned int size, unsigned int dmatag);

/* returns 0 if put is not complete */
unsigned int tnw_test_put(unsigned int dmatag);
void tnw_wait_put(unsigned int dmatag);

unsigned int tnw_get_spe_phys_id();

/* user topology info */



#endif
